 /**************************************************************************
 * Name:      Combobox.cpp                                                 *
 * Purpose:   Code for the Combobox and completion classes                 *
 * Author:    Jackie Team (jackie.team@gmail.com)                          *
 * Created:   2008-02-04                                                   *
 * Copyright: Jackie Team (http://www.musicorganizer.org)                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include "components.h"

BEGIN_EVENT_TABLE(ComboBox, wxComboBox)
    EVT_SET_FOCUS(ComboBox::onGainFocus)
    EVT_KILL_FOCUS(ComboBox::onLooseFocus)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(ComboPath, ComboBox)
    EVT_TEXT_ENTER(wxID_ANY, ComboPath::onEnter)
    EVT_TEXT(wxID_ANY, ComboPath::onText)
    EVT_SET_FOCUS(ComboPath::onGainFocus)
    EVT_KILL_FOCUS(ComboPath::onLooseFocus)
    EVT_KEY_DOWN(ComboPath::onKey)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(completionList, wxListBox)
    EVT_SET_FOCUS(completionList::onGainFocus)
    EVT_KILL_FOCUS(completionList::onLooseFocus)
    #ifndef __WXMAC__
    EVT_LEFT_DOWN(completionList::onClick)
    #else
    EVT_LISTBOX(wxID_ANY,completionList::onClick2)
    #endif
END_EVENT_TABLE()



ComboBox::ComboBox(MonBook* b,wxWindow *parent, wxWindowID id, const wxString& value,
                   const wxPoint& pos, const wxSize& size, const wxArrayString& choices, long style) : wxComboBox(parent,id,value,pos,size,choices,style){
    mb=b;
}

/* ****************************************************************************** */

void ComboBox::addComboBoxItem(wxString chaine){
    if (duplicateMap.count(chaine) > 0){//the item is already in the combobox
        duplicateMap[chaine] += 1; 
    }else{
        duplicateMap[chaine] = 1;
        Append(chaine);
    }
}

/* ****************************************************************************** */

void ComboBox::removeComboBoxItem(wxString chaine){
    if (duplicateMap.count(chaine) > 0){//the item has been added more than one time
        if (duplicateMap[chaine] > 1){
            duplicateMap[chaine] -= 1;
        }else if(duplicateMap[chaine] == 1){//realy remove the item from the combobox
            duplicateMap.erase(chaine);
            Delete(FindString(chaine));
        }
    }
}

/* ****************************************************************************** */

void ComboBox::onGainFocus(wxFocusEvent& e){
    e.Skip();
    mb->getMonChoicebook()->disableMenus();

}

/* ****************************************************************************** */

void ComboBox::onLooseFocus(wxFocusEvent& e){
    e.Skip();
    mb->getMonChoicebook()->disableMenus(false);
}

/* ****************************************************************************** */

void ComboBox::clearAll(){
    Clear();
    duplicateMap.clear();

}

/* ******************************************************************************
   ****************************************************************************** */

ComboPath::ComboPath(MonBook *b,wxWindow *parent, wxWindowID id, const wxString& value,
                   const wxPoint& pos, const wxSize& size, const wxArrayString& choices, long style) : ComboBox(b,parent,id,value,pos,size,choices,style){
      oldPathWritten=_T("");
}

/* ****************************************************************************** */

void ComboPath::setMonArbo(MonArbo *a){
   ma=a;
   pw=NULL;
}

/* ****************************************************************************** */

void ComboPath::onEnter(wxCommandEvent& e){
    //try to set the path of the tree
    if(wxDirExists(GetValue())){//dir realy exist
        //refresh tree
        ma->setIgnore(true);
        ma->SetPath(GetValue());
        ma->setIgnore(false);
        ma->refresh();
   }else{
        SetValue(ma->GetPath());
   }
   if (pw!=NULL)
      pw->Hide();

}

/* ****************************************************************************** */

void ComboPath::onText(wxCommandEvent& e){
    //call when a char is typped
    if(pw==NULL){
        return;
    }
    pw->Show();
    wxString val=GetValue();
    wxFileName fn(val);
    wxString newPathWriting=fn.GetPath(), f(_T(""));
    wxString whatToWrite;
    wxSortedArrayString maybe;
    cl->Clear();

    if(wxDirExists(newPathWriting)){
        if(newPathWriting!=oldPathWritten){// we must redo a scan
            wxDir cur(newPathWriting);
            files.clear();
            DirLister dl(&files);
            cur.Traverse(dl);
            oldPathWritten=newPathWriting;
        }
        for(int i=0 ; i<files.size() ; i++){//fill the completion list
           if(wxFileName(files[i]).GetFullName().StartsWith(fn.GetFullName()) || fn.GetFullName()==_T(""))
               cl->Append(files[i]);
        }
    }
}

/* ****************************************************************************** */

void ComboPath::onGainFocus(wxFocusEvent& e){
     e.Skip();
     wxSize s=GetSize();
     if(pw==NULL){
             //create the completion list
             wxSize s=GetSize();
             #ifdef __WXMAC__
             pw=new wxMiniFrame(this,wxID_ANY,_T(""),wxDefaultPosition,wxDefaultSize,0);
             #else
             pw=new wxPopupWindow(this);
             #endif
             wxPanel* pcl = new wxPanel(pw,wxID_ANY);
             cl=new completionList(this,pcl,wxID_ANY,wxDefaultPosition,wxSize(s.GetWidth(),100),0,NULL,wxLB_SINGLE|wxLB_SORT|wxLB_ALWAYS_SB);
             wxBoxSizer* sil=new wxBoxSizer(wxHORIZONTAL);
             sil->Add(cl,1,wxEXPAND|wxALL);
             sil->Fit(pcl);
             sil->Fit(pw);
             pw->SetSize(GetScreenPosition().x,GetScreenPosition().y+s.GetHeight(),s.GetWidth(),100);
             pw->Hide();
     }else //just show the list
         pw->SetSize(GetScreenPosition().x,GetScreenPosition().y+s.GetHeight(),s.GetWidth(),100);
     #ifndef __WXMAC__
     ComboBox::onGainFocus(e);
     #endif
}

/* ****************************************************************************** */

void ComboPath::onLooseFocus(wxFocusEvent& e){
     e.Skip();
     if(pw!=NULL && (e.GetWindow()==NULL ||
     #ifndef __WXMAC__
       ((completionList*)(e.GetWindow())!=cl && (wxPopupWindow*)(e.GetWindow())!=pw))){
     #else
       ((completionList*)(e.GetWindow())!=cl && (wxMiniFrame*)(e.GetWindow())!=pw))){
     #endif
        //hide the completion list
        pw->Hide();
        #ifndef __WXMAC__
        ComboBox::onLooseFocus(e);
        #endif
     }
}

/* ****************************************************************************** */

void ComboPath::setPath(wxString p){
     SetValue(p);
     if(pw!=NULL){
       cl->Clear();
       pw->Hide();
     }
}

/* ****************************************************************************** */

void ComboPath::onKey(wxKeyEvent &e){
     e.Skip();

    if(e.GetModifiers() == wxMOD_NONE ){
        if(e.GetKeyCode()==WXK_ESCAPE && pw!=NULL)//Escape key hide the list
            pw->Hide();
    }
}


/* ******************************************************************************
   ****************************************************************************** */

completionList::completionList(ComboPath *c, wxWindow* parent, wxWindowID id,wxPoint p, wxSize s,int n,wxString* v,long int f)
                    :wxListBox(parent,id,p,s,n,v,f){
     cp=c;
}

/* ****************************************************************************** */

void completionList::onGainFocus(wxFocusEvent& e){
     e.Skip();
     GetParent()->Show();
}

/* ****************************************************************************** */

void completionList::onLooseFocus(wxFocusEvent& e){
     e.Skip();
     if(e.GetWindow()==NULL && (ComboPath*)(e.GetWindow())!=cp){
          GetParent()->Hide();
     }
}

/* ****************************************************************************** */

void completionList::onClick(wxMouseEvent& e){
     e.Skip();
     int i=HitTest(e.GetPosition());//get the clicked item
     if(i!=wxNOT_FOUND){
        SetSelection(i);
        if(wxDirExists(GetStringSelection())){//update comboPath
            cp->setPath(GetStringSelection());
            cp->SetFocus();
        }
     }
}

/* ****************************************************************************** */

void completionList::onClick2(wxCommandEvent& e){
        if(wxDirExists(GetStringSelection())){
            cp->setPath(GetStringSelection());//update comboPath
            cp->SetFocus();
        }
}
