/***************************************************************************
 * Name:      Mon_Arbo.cpp                                                 *
 * Purpose:   Code for GenericDirCtrl                                      *
 * Author:    Jackie Team (jackie.team@gmail.com)                          *
 * Created:   2007-11-26                                                   *
 * Copyright: Jackie Team (www.musicorganizer.org)                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include "components.h"

IMPLEMENT_DYNAMIC_CLASS(MonArbo, wxGenericDirCtrl)

BEGIN_EVENT_TABLE(MonArbo, wxGenericDirCtrl)
    EVT_TREE_SEL_CHANGED(wxID_ANY,MonArbo::change)
    EVT_MENU(THREAD,MonArbo::endChange)
    EVT_MENU(ID_DROPTHREAD,MonArbo::dropThreadMsg)
END_EVENT_TABLE()

MonArbo::MonArbo(wxWindow* parent,MonChoicebook* c, MonBook* b) : wxGenericDirCtrl(parent, wxID_ANY,_T(""),wxPoint(0,0),wxDefaultSize,wxDIRCTRL_DIR_ONLY|wxDIRCTRL_EDIT_LABELS)
{
    cb=c;
    book=b;
    wait=new WaitPopup(this);
    ignore_event=false;
    wxString savedPath=OptionsHandler::getOption(wxString(wxT("/tree/path")));
    //the folder is changed if wanted but not scanned
    if(wxDirExists(savedPath) && OptionsHandler::getOption(wxString(wxT("/tree/save_dir")),true)){
        setIgnore(true);
        SetPath(savedPath);
        setIgnore(false);
    }
    oldindex=OptionsHandler::getOption(_T("/tree/history/index"),0);
    nextindex=-1;
}

/* ********************************************************************************* */

void MonArbo::setIgnore(bool b){
   ignore_event=b;
}

/* ********************************************************************************* */

void MonArbo::historyGo(int d){
    if(OptionsHandler::getOption(_T("/tree/has_history"),true)){//we update history
        int where=OptionsHandler::getOption(_T("/tree/history/index"),0);
        nextindex=where+d;
        wxString path=wxString::Format(_T("/tree/history/path%d"),where+d);

        if(wxDirExists(OptionsHandler::getOption(path))){
            //we must do this hack for wxWindows
            setIgnore(true);
            SetPath(OptionsHandler::getOption(path));
            setIgnore(false);
            refresh();
        }
    }
}

/* ********************************************************************************* */

void MonArbo::change(wxTreeEvent& e){	
    if(ignore_event==true)
        return;
    if(OptionsHandler::getOption(_T("/tree/has_history"),true)){//history is updated
        int where=OptionsHandler::getOption(_T("/tree/history/index"),0);

        if(nextindex!=-1){//change from menu
           if(oldindex!=nextindex){//their was a real changing
               OptionsHandler::setOption(wxString::Format(_T("/tree/history/path%d"),nextindex),GetPath());
               OptionsHandler::setOption(_T("/tree/history/index"),nextindex);
               oldindex=nextindex;
               wxCommandEvent e( wxEVT_COMMAND_MENU_SELECTED, MusicOrganizer::ID_HIST);
               e.SetInt(-1);
               wxPostEvent( GetParent(), e );
            }
        }else if(OptionsHandler::getOption(wxString::Format(_T("/tree/history/path%d"),where))!=GetPath()){
            if(where==10){//the stack is full
                 for(int i=1 ; i<10 ; i++){//make a shift
                     wxString newVal=OptionsHandler::getOption(wxString::Format(_T("/tree/history/path%d"),i+1));
                     OptionsHandler::setOption(wxString::Format(_T("/tree/history/path%d"),i),newVal);
                 }
                 where--;
            }else if(OptionsHandler::getOption(wxString::Format(_T("/tree/history/path%d"),where+1))!=GetPath()){
                //then we must clear the nexts
                for(int i=where+1;i<=10 ; i++)
                    OptionsHandler::setOption(wxString::Format(_T("/tree/history/path%d"),i),wxString(_T("")));
            }

            OptionsHandler::setOption(wxString::Format(_T("/tree/history/path%d"),where+1),GetPath());
            OptionsHandler::setOption(_T("/tree/history/index"),where+1);
            oldindex=where+1;
            wxCommandEvent e( wxEVT_COMMAND_MENU_SELECTED, MusicOrganizer::ID_HIST);
            e.SetInt(-1);
            wxPostEvent( GetParent(), e );
        }
    }
    nextindex=-1;//reset the value
    //Saving the path
    OptionsHandler::setOption(_T("/tree/path"),GetPath());

    //clean combobox
    book->getComboTitre()->Clear();
    book->getComboInterprete()->Clear();
    book->getComboAlbum()->Clear();
    book->getComboPiste()->Clear();
    book->getComboAlbum()->SetValue(_T(""));
    book->getComboTitre()->SetValue(_T(""));
    book->getComboInterprete()->SetValue(_T(""));
    book->getComboAnne()->SetValue(_T(""));
    book->getComboGenre()->SetValue(_T(""));
    book->getComboPiste()->SetValue(_T(""));
    book->getComboAlbum2()->SetValue(_T(""));
    book->getComboTitre2()->SetValue(_T(""));
    book->getComboInterprete2()->SetValue(_T(""));
    book->getComboAnne2()->SetValue(_T(""));
    book->getComboGenre2()->SetValue(_T(""));
    book->getComboPiste2()->SetValue(_T(""));
    //clean list
    cb->getMListeV1()->removeColumnImage();
    cb->getMListeV2()->removeColumnImage();

    wxString path;
    path=GetPath();


    wait->Centre();
    wait->Show();
    files.Clear();
    Disable();//when scanning the user must not be able to change the folder (to avoid multiple scanning at the same time)
    //create thread
    bool recursive=OptionsHandler::getOption(_T("/tree/recursive"),true);
    if (path.size()==3 and path.substr(1,3)==_T(":\\") or path==_T("/"))
        t=new ScanThread(this,GetPath(),&files,false); //root directory, or drive root directory for windows
    else
        t=new ScanThread(this,GetPath(),&files,recursive);

    t->Create();
    t->Run();

}

/* ********************************************************************************* */

void MonArbo::endChange(wxCommandEvent& event){
    bool arreter=false;
    wait->Hide();
    std::sort( files.begin(), files.end() );
    cb->getMListeV1()->DeleteAllItems();
    cb->getMListeV2()->DeleteAllItems();
    wxProgressDialog* progBar=new wxProgressDialog(_("Please wait"),_("Informations reading..."),100,NULL,wxPD_AUTO_HIDE | wxPD_APP_MODAL | wxPD_CAN_ABORT /*| wxPD_SMOOTH*/);

    for (unsigned int i=0 ; i<files.Count() ; i++){

        wxString file=files[i];
        arreter=!progBar->Update((i*1.0)/files.size()*100,_T(""));
        if(arreter==true)
            break;
        cb->readAndShowInfos(file,i);
    }
    if(files.size()==0){
         cb->getMListeV2()->sendMenuMsg(-1);
         cb->getMListeV1()->sendMenuMsg(-1);
    }else{
         cb->getMListeV2()->sendMenuMsg(2);
         cb->getMListeV1()->sendMenuMsg(2);
    }
    progBar->Destroy();
    Enable();
}

/* ********************************************************************************* */

void MonArbo::refresh(){
        wxString path;
        path=GetPath();
        wxTreeEvent e(wxEVT_COMMAND_TREE_SEL_CHANGED,wxID_ANY);
        setIgnore(true);//to avoid double scanning
        CollapseTree();
        SetPath(path);
        setIgnore(false);
        wxPostEvent(this,e);
}

/* ********************************************************************************* */

void MonArbo::stopThread(){
    t->Stop();
}

/* ********************************************************************************* */

void MonArbo::SetPath(const wxString& path){
    //Default SetPath change cwd, not usefull for us
    wxString cwd=wxFileName::GetCwd();
    wxGenericDirCtrl::SetPath(path);
    wxFileName::SetCwd(cwd);
}

/* ********************************************************************************* */

void MonArbo::dropThreadMsg(wxCommandEvent& event){
    wxString type=event.GetString();

    if(type==_T("up")){//progbar update
        if(!progBar->Update(event.GetInt()))
            cdt->end();

    }else{
        int errors=event.GetInt();
        if (errors != 0){
            if (errors == 1){
                wxMessageDialog msg(NULL,_("1 error while performing task"), _("Error"),wxICON_HAND|wxOK);
                msg.ShowModal();
            }else{
                wxString err=wxString::Format(_T("%d"),errors);
                wxMessageDialog msg(NULL,err + _(" errors while performing task"), _("Error"),wxICON_HAND|wxOK);
                msg.ShowModal();
            }
        }

        refresh();
    }
}


/* *********************************************************************************
   ********************************************************************************* */
DropTargetCopie::DropTargetCopie(MonArbo* a){
      ma=a;
      oldid=ma->GetTreeCtrl()->GetRootItem();
}

/* ********************************************************************************* */

bool DropTargetCopie::OnDropText(wxCoord x, wxCoord y,const wxString& data){
      //some files have been droped
      wxPoint p(x,y);

      wxTreeItemId root=ma->GetTreeCtrl()->GetRootItem();
      wxTreeItemId tid=ma->GetTreeCtrl()->HitTest(p);
      //Get the path of tid
      wxString path=ma->GetTreeCtrl()->GetItemText(tid);
      while((tid=ma->GetTreeCtrl()->GetItemParent(tid))!=root){
          path=ma->GetTreeCtrl()->GetItemText(tid)+wxFileName::GetPathSeparator()+path;
      }

      wxMessageDialog dlg_info(NULL,_("Copy file(s) in ")+path,_("Warning!"), wxICON_EXCLAMATION | wxYES_NO);
      int res=dlg_info.ShowModal();
       dlg_info.Destroy();

      if (res==wxID_NO)
        return false;

      //copy files using a thread
      ma->setProgBar(new wxProgressDialog(_("Please wait"),_("Files are beeing copied..."),100,NULL,wxPD_AUTO_HIDE | wxPD_CAN_ABORT | wxPD_APP_MODAL));
      ma->setCdt(new CopyDropThread(ma,data,path));
      ma->getCdt()->Create();
      ma->getCdt()->Run();

      return true;
}

/* ********************************************************************************* */

wxDragResult DropTargetCopie::OnDragOver(wxCoord x, wxCoord y, wxDragResult def){
      wxPoint p(x,y);
      wxTreeCtrl *tc=ma->GetTreeCtrl();

      wxTreeItemId tid=tc->HitTest(p);//the id of the item under mouse
      if(oldid!=tc->GetRootItem() and oldid!=tid){//not default and it has changed
          tc->SetItemBold(oldid,false);
      }
      tc->SetItemBold(tid,true);

      oldid=tid;

      return def;
}

/* ********************************************************************************* */

void DropTargetCopie::OnLeave(){
      if(oldid!=ma->GetTreeCtrl()->GetRootItem()){//if it's not the default id
          ma->GetTreeCtrl()->SetItemBold(oldid,false);
      }
      oldid=ma->GetTreeCtrl()->GetRootItem();
}
