/***************************************************************************
 *   Copyright (C) 2007 by Jackie Team                                     * 
 *   jackie.team@gmail.com                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//taglib includes
#include <fileref.h>
#include <tag.h>
#include <id3v1tag.h>
#include <id3v2tag.h>
#include <mpegfile.h>

#include <wx/dynarray.h>
#include "components.h"


BEGIN_EVENT_TABLE(MonChoicebook, wxChoicebook)
    EVT_CHOICEBOOK_PAGE_CHANGED(wxID_ANY, MonChoicebook::onChanged)
END_EVENT_TABLE()

    MonChoicebook::MonChoicebook(wxWindow* parent,MonBook* b):wxChoicebook(parent,wxID_ANY){
        book=b;
        wxPanel* panelPage1 = new wxPanel(this);
        wxPanel* panelPage2 = new wxPanel(this);
        mlisteV1=new MaListe(panelPage1,b,this);
        mlisteV2=new MaListe(panelPage2,b,this);
        AddPage(panelPage2,_T("ID3 V2"),true);
        AddPage(panelPage1,_T("ID3 V1"),false);

        wxBoxSizer* sizerV1=new wxBoxSizer(wxHORIZONTAL),*sizerV2=new wxBoxSizer(wxHORIZONTAL);

        sizerV1->Add(mlisteV1,1,wxEXPAND|wxALL);
        panelPage1->SetSizer(sizerV1);
        sizerV2->Add(mlisteV2,1,wxEXPAND|wxALL);
        panelPage2->SetSizer(sizerV2);
    }

/* ****************************************************************************** */

    MaListe* MonChoicebook::getMListeV1(){
        return mlisteV1;
    }

/* ****************************************************************************** */

    MaListe* MonChoicebook::getMListeV2(){
        return mlisteV2;
    }

/* ****************************************************************************** */

    MaListe* MonChoicebook::getCurrentList(){
        if(GetSelection() == 0) return mlisteV2;
        else return mlisteV1;
    }

/* ****************************************************************************** */

    MaListe* MonChoicebook::getOtherList(){
        if(GetSelection() == 0) return mlisteV1;
        else return mlisteV2;
    }

/* ****************************************************************************** */

    void MonChoicebook::selectItem(long index,MaListe* who,bool sel){
        wxListItem current;
        //retrieve columns content of the selected item
        wxString t,i,a,p,tv1,iv1,av1,pv1;
        current.m_itemId = index;
        current.SetMask(wxLIST_MASK_TEXT);
        //ID3V2 tags
        current.SetColumn(MaListe::TITLE_COL);
        mlisteV2->GetItem(current);
        t=current.GetText();
        current.SetColumn(MaListe::ARTIST_COL);
        mlisteV2->GetItem(current);
        i=current.GetText();
        current.SetColumn(MaListe::ALBUM_COL);
        mlisteV2->GetItem(current);
        a=current.GetText();

        //ID3V1 tags
        current.SetColumn(MaListe::TITLE_COL);
        mlisteV1->GetItem(current);
        tv1=current.GetText();
        current.SetColumn(MaListe::ARTIST_COL);
        mlisteV1->GetItem(current);
        iv1=current.GetText();
        current.SetColumn(MaListe::ALBUM_COL);
        mlisteV1->GetItem(current);
        av1=current.GetText();


        if(sel){//select item
            if(who==mlisteV1){
                 mlisteV2->setEnableEvent(false);
                 mlisteV2->SetItemState(index,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
                 mlisteV2->setEnableEvent(true);
            }else{
                 mlisteV1->setEnableEvent(false);
                 mlisteV1->SetItemState(index,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
                 mlisteV1->setEnableEvent(true);
            }
            fill_combo();

            //add not empty tags to the combobox
            //id3V2
            if (t != _T(""))
                book->getComboTitre()->addComboBoxItem(t);

            if (i != _T(""))
                book->getComboInterprete()->addComboBoxItem(i);

            if (a != _T(""))
                book->getComboAlbum()->addComboBoxItem(a);

            //id3V1
            if (tv1 != _T(""))
                book->getComboTitre2()->addComboBoxItem(tv1);

            if (iv1 != _T(""))
                book->getComboInterprete2()->addComboBoxItem(iv1);

            if (av1 != _T(""))
                book->getComboAlbum2()->addComboBoxItem(av1);
       }else{//deselect item
            if(who==mlisteV1){
                 mlisteV2->setEnableEvent(false);
                 mlisteV2->SetItemState(index,0,wxLIST_STATE_SELECTED);
                 mlisteV2->setEnableEvent(true);
            }else{
                 mlisteV1->setEnableEvent(false);
                 mlisteV1->SetItemState(index,0,wxLIST_STATE_SELECTED);
                 mlisteV1->setEnableEvent(true);
            }
            fill_combo();

            //remove not empty tags from the combobox
            //id3v2
            if (t != _T(""))
                book->getComboTitre()->removeComboBoxItem(t);

            if (i != _T(""))
                book->getComboInterprete()->removeComboBoxItem(i);

            if (a != _T(""))
                book->getComboAlbum()->removeComboBoxItem(a);

            //id3v1
            if (tv1 != _T(""))
                book->getComboTitre2()->removeComboBoxItem(tv1);

            if (iv1 != _T(""))
                book->getComboInterprete2()->removeComboBoxItem(iv1);

            if (av1 != _T(""))
                book->getComboAlbum2()->removeComboBoxItem(av1);
       }
    }

/* ****************************************************************************** */

   void MonChoicebook::delCurrent(){
       long ancienneSel = mlisteV1->getSelection(-2);
       while(mlisteV1->GetSelectedItemCount() > 0){//for each selected files
           long index = mlisteV1->FindItem(-1,mlisteV1->getSelRowText(0,ancienneSel));
           mlisteV1->SetItemState(index,0,wxLIST_STATE_SELECTED);
           mlisteV1->DeleteItem(index);
           mlisteV2->DeleteItem(index);

           ancienneSel = mlisteV1->getSelection(ancienneSel) - 1;
       }
       if(mlisteV1->GetItemCount()==0)//if there is no more item in the list
            mlisteV1->sendMenuMsg(-1);//update menus
   }

/* ****************************************************************************** */

    void MonChoicebook::readAndShowInfos(wxString f,int index){
        wxFileName fn(f);
        wxString name=fn.GetFullName();;
        wxString directory=fn.GetPath();

        name  = wxString(name.fn_str(),*wxConvCurrent);
        #ifdef __WXMSW__ //strange encoding bug 
        TagLib::MPEG::File fic((const char*)f.mb_str(wxConvUTF8),true,TagLib::AudioProperties::Average);
        #else
        TagLib::MPEG::File fic((const char*)f.fn_str(),true,TagLib::AudioProperties::Average);
        #endif

        //read ID3V2 tags of the file
        TagLib::String artist(""),titre(""),album(""),genre("");
        TagLib::uint track(atoi("")),year(atoi(""));
        if(fic.ID3v2Tag()!=NULL){//ID3V2tag
            artist = fic.ID3v2Tag()->artist();
            titre = fic.ID3v2Tag()->title();
            album = fic.ID3v2Tag()->album();
            genre = fic.ID3v2Tag()->genre();
            track = fic.ID3v2Tag()->track();
            year = fic.ID3v2Tag()->year();
        }

        int length = fic.audioProperties()->length();

        int heure=0,minute=0,seconde=0;
        wxString duree;
        //create a human readable string of the length...
        //...first compute hours, minutes and seconds...
        if(length>=3600){
            heure=length/3600;
            minute=(length-heure*3600)/60;
            seconde=length-heure*3600-minute*60;
        }else if (length>=60){
            minute=length/60;
            seconde=length-minute*60;
        }else
            seconde=length;
        //...then build the string
        if(heure!=0){
           if (minute <10){
              if (seconde<10)
                  duree.Printf(_T("%d:0%d:0%d"),heure,minute,seconde);
              else
                  duree.Printf(_T("%d:0%d:%d"),heure,minute,seconde);
           }else{
              if (seconde<10)
                  duree.Printf(_T("%d:%d:0%d"),heure,minute,seconde);
              else
                  duree.Printf(_T("%d:%d:%d"),heure,minute,seconde);
           }
        }else if (minute!=0){
           if (seconde<10)
               duree.Printf(_T("%d:0%d"),minute,seconde);
           else
               duree.Printf(_T("%d:%d"),minute,seconde);
        }else{
            if (seconde<10)
                duree.Printf(_T("0:0%d"),seconde);
            else
                duree.Printf(_T("0:%d"),seconde);
        }

        //convert the taglib strings to wxStrings
        wxString ar(artist.toCString(true),wxConvUTF8);
        wxString t(titre.toCString(true),wxConvUTF8);
        wxString al(album.toCString(true),wxConvUTF8);
        wxString g(genre.toCString(true),wxConvUTF8);

        wxString tr,y;
        if(track!=0)
            tr.Printf(_T("%u"),track);
        if(year!=0)
            y.Printf(_T("%u"),year);

        //read ID3V1 tags
        wxString arV1,tV1,alV1,gV1,yV1,trV1;
        #ifdef __WXMSW__
        TagLib::MPEG::File fv1((const char*)f.mb_str(wxConvUTF8),true,TagLib::AudioProperties::Average);
        #else
        TagLib::MPEG::File fv1((const char*)f.fn_str(),true,TagLib::AudioProperties::Average);
        #endif
        if(fv1.ID3v1Tag()!=NULL){
            trV1.Printf(_T("%u"),fv1.ID3v1Tag()->track());
            yV1.Printf(_T("%u"),fv1.ID3v1Tag()->year());
            arV1=wxString(fv1.ID3v1Tag()->artist().toCString(true),wxConvUTF8);
            tV1=wxString(fv1.ID3v1Tag()->title().toCString(true),wxConvUTF8);
            alV1=wxString(fv1.ID3v1Tag()->album().toCString(true),wxConvUTF8);
            gV1=wxString(fv1.ID3v1Tag()->genre().toCString(true),wxConvUTF8);
            if(trV1==_T("0")) trV1=_T("");
            if(yV1==_T("0")) yV1=_T("");
        }
        //clean tags
        name.Replace(_T("\0"),_T(""));
        ar.Replace(_T("\0"),_T(""));
        al.Replace(_T("\0"),_T(""));
        y.Replace(_T("\0"),_T(""));
        g.Replace(_T("\0"),_T(""));
        tr.Replace(_T("\0"),_T(""));
        t.Replace(_T("\0"),_T(""));

        arV1.Replace(_T("\0"),_T(""));
        alV1.Replace(_T("\0"),_T(""));
        yV1.Replace(_T("\0"),_T(""));
        gV1.Replace(_T("\0"),_T(""));
        trV1.Replace(_T("\0"),_T(""));
        tV1.Replace(_T("\0"),_T(""));

        //insert the new item and fill the columns
        mlisteV1->InsertItem(index,name);
        #ifdef __WXMSW__
        mlisteV1->SetItemImage(index,-1);
        #endif
        mlisteV1->SetItemData(index,index);
        mlisteV1->SetItem(index,MaListe::TITLE_COL,tV1);
        mlisteV1->SetItem(index,MaListe::ARTIST_COL,arV1);
        mlisteV1->SetItem(index,MaListe::ALBUM_COL,alV1);
        mlisteV1->SetItem(index,MaListe::YEAR_COL,yV1);
        mlisteV1->SetItem(index,MaListe::GENRE_COL,gV1);
        mlisteV1->SetItem(index,MaListe::TRACK_COL,trV1);
        mlisteV1->SetItem(index,MaListe::LEN_COL,duree);
        mlisteV1->SetItem(index,MaListe::DIR_COL,directory);
        mlisteV2->InsertItem(index,name);
        #ifdef __WXMSW__
        mlisteV2->SetItemImage(index,-1);
        #endif
        mlisteV2->SetItemData(index,index);
        mlisteV2->SetItem(index,MaListe::TITLE_COL,t);
        mlisteV2->SetItem(index,MaListe::ARTIST_COL,ar);
        mlisteV2->SetItem(index,MaListe::ALBUM_COL,al);
        mlisteV2->SetItem(index,MaListe::YEAR_COL,y);
        mlisteV2->SetItem(index,MaListe::GENRE_COL,g);
        mlisteV2->SetItem(index,MaListe::TRACK_COL,tr);
        mlisteV2->SetItem(index,MaListe::LEN_COL,duree);
        mlisteV2->SetItem(index,MaListe::DIR_COL,directory);

        //define the background colour of the item
        wxColour blanc = wxColour(255,255,255);
        wxColour grisClair = wxColour(240,240,240);
        wxColour secColor = wxColour(206,235,253);
        wxColour secColor2 = wxColour(230,248,254);
        if (index == 0){//first item always have a white background
            mlisteV1->SetItemBackgroundColour(index,blanc);
            mlisteV2->SetItemBackgroundColour(index,blanc);
        }else if (mlisteV2->getRowText(index,MaListe::DIR_COL) == mlisteV2->getRowText(index-1,MaListe::DIR_COL)){
            if (mlisteV2->GetItemBackgroundColour(index-1) == blanc){
                mlisteV1->SetItemBackgroundColour(index,grisClair);
                mlisteV2->SetItemBackgroundColour(index,grisClair);
            }else if (mlisteV2->GetItemBackgroundColour(index-1) == grisClair){
                mlisteV1->SetItemBackgroundColour(index,blanc);
                mlisteV2->SetItemBackgroundColour(index,blanc);
            }else if (mlisteV2->GetItemBackgroundColour(index-1) == secColor){
                mlisteV1->SetItemBackgroundColour(index,secColor2);
                mlisteV2->SetItemBackgroundColour(index,secColor2);
            }else{
                mlisteV1->SetItemBackgroundColour(index,secColor);
                mlisteV2->SetItemBackgroundColour(index,secColor);
            }
        }else{
            if (mlisteV2->GetItemBackgroundColour(index-1) == blanc || mlisteV2->GetItemBackgroundColour(index-1) == grisClair){
                mlisteV1->SetItemBackgroundColour(index,secColor);
                mlisteV2->SetItemBackgroundColour(index,secColor);
            }else{
                mlisteV1->SetItemBackgroundColour(index,blanc);
                mlisteV2->SetItemBackgroundColour(index,blanc);
            }
        }
    }

/* ****************************************************************************** */

    void MonChoicebook::sortLists(int& currentCol,int oldCol,bool desc,MaListe* who){
        MaListe* other;
        if(who==mlisteV1) other=mlisteV2;
        else other=mlisteV1;

        wxListItem item;
        item.SetMask(wxLIST_MASK_IMAGE);
        item.SetImage(-1);
        who->SetColumn(currentCol, item);//delete picture
        other->SetColumn(currentCol, item);//delete picture
        currentCol = oldCol;
        other->setCurCol(currentCol);
        if(desc)
            item.SetImage(1);//down
        else
            item.SetImage(0);//up
        who->SetColumn(currentCol, item);
        other->SetColumn(currentCol, item);

        //the array which will contain all tags
        wxArrayString nom;
        wxArrayString titre;
        wxArrayString artiste;
        wxArrayString album;
        wxArrayString annee;
        wxArrayString genre;
        wxArrayString piste;
        wxArrayString duree;
        wxArrayString othertmp[6];
        wxArrayString chemin;
        ArrayOfInts states;

        //std::multimap are used because it sort automaticly and fastly data
        std::multimap<wxString, int>::iterator it;
        std::multimap<wxString, int>::reverse_iterator rit;
        std::multimap<wxString, int> myMap;

        bool sortByFolder = OptionsHandler::getOption(_T("/list/sort_by_dir"),true);
        for (long i=0;i<who->GetItemCount();i++){//for each item
            nom.Add(who->getRowText(i,MaListe::NAME_COL));
            titre.Add(who->getRowText(i,MaListe::TITLE_COL));
            artiste.Add(who->getRowText(i,MaListe::ARTIST_COL));
            album.Add(who->getRowText(i,MaListe::ALBUM_COL));
            annee.Add(who->getRowText(i,MaListe::YEAR_COL));
            genre.Add(who->getRowText(i,MaListe::GENRE_COL));
            piste.Add(who->getRowText(i,MaListe::TRACK_COL));
            duree.Add(who->getRowText(i,MaListe::LEN_COL));
            chemin.Add(who->getRowText(i,MaListe::DIR_COL));
            //retreive tags of unselected list
            for(int k=0; k<6 ; k++)
                othertmp[k].Add(other->getRowText(i,k+1));

            long val;
            if(sortByFolder && currentCol != MaListe::DIR_COL){
                if(currentCol == MaListe::TRACK_COL || currentCol==MaListe::YEAR_COL){//fix number sorting
                    who->getRowText(i,currentCol).ToLong(&val);
                    myMap.insert(std::pair<wxString, int>(who->getRowText(i,MaListe::DIR_COL) + wxString::Format(_T("%04d"),val),i));
                }else
                    myMap.insert(std::pair<wxString, int>(who->getRowText(i,MaListe::DIR_COL) + who->getRowText(i,currentCol),i));
            }else{
                if(currentCol == MaListe::TRACK_COL || currentCol==MaListe::YEAR_COL){//fix number sorting
                    who->getRowText(i,currentCol).ToLong(&val);
                    myMap.insert(std::pair<wxString, int>(wxString::Format(_T("%04d"),val),i));
                }else
                    myMap.insert(std::pair<wxString, int>(who->getRowText(i,currentCol),i));
            }
        }
        long i=0;
        //refill the lists
        if(!desc){//increasing order
            for ( it=myMap.begin(); it != myMap.end(); it++ ){//for each item
                states.Add(who->GetItemState(it->second,wxLIST_STATE_SELECTED));
                who->SetItem(i,MaListe::NAME_COL,nom.Item(it->second));
                other->SetItem(i,MaListe::NAME_COL,nom.Item(it->second));
                who->SetItem(i,MaListe::TITLE_COL,titre.Item(it->second));
                who->SetItem(i,MaListe::ARTIST_COL,artiste.Item(it->second));
                who->SetItem(i,MaListe::ALBUM_COL,album.Item(it->second));
                who->SetItem(i,MaListe::YEAR_COL,annee.Item(it->second));
                who->SetItem(i,MaListe::GENRE_COL,genre.Item(it->second));
                who->SetItem(i,MaListe::TRACK_COL,piste.Item(it->second));
                who->SetItem(i,MaListe::LEN_COL,duree.Item(it->second));
                other->SetItem(i,MaListe::LEN_COL,duree.Item(it->second));
                who->SetItem(i,MaListe::DIR_COL,chemin.Item(it->second));
                other->SetItem(i,MaListe::DIR_COL,chemin.Item(it->second));
                for(int k=0; k<6 ; k++)
                   other->SetItem(i,k+1,othertmp[k][it->second]);
                i++;
            }
        }else{//decreasing order
            for ( rit=myMap.rbegin(); rit != myMap.rend(); rit++ ){//for each item
                states.Add(who->GetItemState(rit->second,wxLIST_STATE_SELECTED));
                who->SetItem(i,MaListe::NAME_COL,nom.Item(rit->second));
                other->SetItem(i,MaListe::NAME_COL,nom.Item(rit->second));
                who->SetItem(i,MaListe::TITLE_COL,titre.Item(rit->second));
                who->SetItem(i,MaListe::ARTIST_COL,artiste.Item(rit->second));
                who->SetItem(i,MaListe::ALBUM_COL,album.Item(rit->second));
                who->SetItem(i,MaListe::YEAR_COL,annee.Item(rit->second));
                who->SetItem(i,MaListe::GENRE_COL,genre.Item(rit->second));
                who->SetItem(i,MaListe::TRACK_COL,piste.Item(rit->second));
                who->SetItem(i,MaListe::LEN_COL,duree.Item(rit->second));
                other->SetItem(i,MaListe::LEN_COL,duree.Item(rit->second));
                who->SetItem(i,MaListe::DIR_COL,chemin.Item(rit->second));
                other->SetItem(i,MaListe::DIR_COL,chemin.Item(rit->second));
                for(int k=0; k<6 ; k++)
                   other->SetItem(i,k+1,othertmp[k][rit->second]);
                i++;
            }
        }
        //Recoloration and reselection
        for (i=0;i<who->GetItemCount();i++){
            who->SetItemState(i,states[i],wxLIST_STATE_SELECTED);
            other->setEnableEvent(false);
            other->SetItemState(i,states[i],wxLIST_STATE_SELECTED);
            other->setEnableEvent(true);
            wxColour blanc = wxColour(255,255,255);
            wxColour grisClair = wxColour(240,240,240);
            wxColour secColor = wxColour(206,235,253);
            wxColour secColor2 = wxColour(230,248,254);
            if (i == 0){
               who->SetItemBackgroundColour(i,blanc);
               other->SetItemBackgroundColour(i,blanc);
            }else if (who->getRowText(i,MaListe::DIR_COL) == who->getRowText(i-1,MaListe::DIR_COL)){
                if (who->GetItemBackgroundColour(i-1) == blanc){
                    who->SetItemBackgroundColour(i,grisClair);
                    other->SetItemBackgroundColour(i,grisClair);
                }else if (who->GetItemBackgroundColour(i-1) == grisClair){
                    who->SetItemBackgroundColour(i,blanc);
                    other->SetItemBackgroundColour(i,blanc);
                }else if (who->GetItemBackgroundColour(i-1) == secColor){
                    who->SetItemBackgroundColour(i,secColor2);
                    other->SetItemBackgroundColour(i,secColor2);
                }else{
                    who->SetItemBackgroundColour(i,secColor);
                    other->SetItemBackgroundColour(i,secColor);
                }
            }else{
                if (who->GetItemBackgroundColour(i-1) == blanc || who->GetItemBackgroundColour(i-1) == grisClair){
                    who->SetItemBackgroundColour(i,secColor);
                    other->SetItemBackgroundColour(i,secColor);
                }else{
                    who->SetItemBackgroundColour(i,blanc);
                    other->SetItemBackgroundColour(i,blanc);
                }
            }
        }
    }

/* ****************************************************************************** */

    void MonChoicebook::onChanged(wxChoicebookEvent& e){
         //the displayed list have been changed
         wxRect rectV1;
         wxRect rectV2;
         //scroll the new displayed list
         if(GetSelection()==0){//ID3V2 list is displayed now
             #ifdef __WXMSW__
             rectV1 = mlisteV1->GetViewRect();
             rectV2 = mlisteV2->GetViewRect();
             //use correct function, but only available for this platform (sick...)
             mlisteV2->ScrollList(0-rectV1.GetX()+rectV2.GetX(),0-rectV1.GetY()+rectV2.GetY());
             #else 
             int y = mlisteV1->GetScrollPos(wxVERTICAL);
             int x = mlisteV1->GetScrollPos(wxHORIZONTAL);
             wxSize s=mlisteV2->GetSize();
             mlisteV2->SetScrollPos(wxVERTICAL, y);
             mlisteV2->SetScrollPos(wxHORIZONTAL, x);
             //an ugly hack to force scrollbar and list to be redrawn
             mlisteV2->SetSize(s.x-1,s.y-1);
             mlisteV2->SetSize(s);
             #endif
             book->SetPageText(3,_T("ID3V2->ID3V1"));
             for(int i=0 ; i<mlisteV1->GetColumnCount() ; i++){
                    mlisteV2->SetColumnWidth(i,mlisteV1->GetColumnWidth(i));
             }
         }else{//ID3V1 list is displayed now
             #ifdef __WXMSW__
             rectV1 = mlisteV1->GetViewRect();
             rectV2 = mlisteV2->GetViewRect();
             mlisteV1->ScrollList(0-rectV2.GetX()+rectV1.GetX(),0-rectV2.GetY()+rectV1.GetY());
             #else
             mlisteV1->SetScrollPos(wxVERTICAL, mlisteV2->GetScrollPos(wxVERTICAL));
             mlisteV1->SetScrollPos(wxHORIZONTAL, mlisteV2->GetScrollPos(wxHORIZONTAL));
             wxSize s=mlisteV1->GetSize();
             mlisteV1->SetSize(s.x-1,s.y-1);
             mlisteV1->SetSize(s);
             #endif
             book->SetPageText(3,_T("ID3V1->ID3V2"));
             for(int i=0 ; i<mlisteV2->GetColumnCount() ; i++){
                    mlisteV1->SetColumnWidth(i,mlisteV2->GetColumnWidth(i));
             }
         }
    }

 /* ****************************************************************************** */

   bool MonChoicebook::isID3V2(){
        if(GetSelection() == 0) return true;
        else return false;
    }

/* ****************************************************************************** */

    void MonChoicebook::fill_combo(){
        if (mlisteV2->GetSelectedItemCount() == 1){//If only on row is selected
            book->getComboAlbum()->SetValue(mlisteV2->getSelRowText(MaListe::ALBUM_COL,-1));
            book->getComboTitre()->SetValue(mlisteV2->getSelRowText(MaListe::TITLE_COL,-1));
            book->getComboInterprete()->SetValue(mlisteV2->getSelRowText(MaListe::ARTIST_COL,-1));
            book->getComboAnne()->SetValue(mlisteV2->getSelRowText(MaListe::YEAR_COL,-1));
            book->getComboGenre()->SetValue(mlisteV2->getSelRowText(MaListe::GENRE_COL,-1));
            book->getComboPiste()->SetValue(mlisteV2->getSelRowText(MaListe::TRACK_COL,-1));
            book->getComboAlbum2()->SetValue(mlisteV1->getSelRowText(MaListe::ALBUM_COL,-1));
            book->getComboTitre2()->SetValue(mlisteV1->getSelRowText(MaListe::TITLE_COL,-1));
            book->getComboInterprete2()->SetValue(mlisteV1->getSelRowText(MaListe::ARTIST_COL,-1));
            book->getComboAnne2()->SetValue(mlisteV1->getSelRowText(MaListe::YEAR_COL,-1));
            book->getComboGenre2()->SetValue(mlisteV1->getSelRowText(MaListe::GENRE_COL,-1));
            book->getComboPiste2()->SetValue(mlisteV1->getSelRowText(MaListe::TRACK_COL,-1));
        }else if(mlisteV2->GetSelectedItemCount() != 0){
            //Inialize combobox
            book->getComboAlbum()->SetValue(_("<Album>"));
            book->getComboTitre()->SetValue(_("<Title>"));
            book->getComboInterprete()->SetValue(_("<Artist>"));
            book->getComboAnne()->SetValue(_("<Year>"));
            book->getComboPiste()->SetValue(_("<Track>"));
            book->getComboGenre()->SetValue(_("<Genre>"));
            book->getComboAlbum2()->SetValue(_("<Album>"));
            book->getComboTitre2()->SetValue(_("<Title>"));
            book->getComboInterprete2()->SetValue(_("<Artist>"));
            book->getComboAnne2()->SetValue(_("<Year>"));
            book->getComboPiste2()->SetValue(_("<Track>"));
            book->getComboGenre2()->SetValue(_("<Genre>"));
        }
    }

/* ****************************************************************************** */

    void MonChoicebook::disableMenus(bool d){
       if(d){
           mlisteV1->sendMenuMsg(-1);//disable all items
       }else{//enable only usable item
           if(mlisteV1->GetItemCount()>0){
               if(mlisteV1->GetSelectedItemCount()>0)
                   mlisteV1->sendMenuMsg(0);
               else
                   mlisteV1->sendMenuMsg(2);
           }
       }
    }
