/***************************************************************************
 *   Copyright (C) 2007 by Jackie Team   *
 *   jackie.team@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "options_Dialog.h"
#include "images/icon.xpm"
#include "images/op_gen.xpm"
#include "images/op_adv.xpm"

#if 0//def __WXMAC__
namespace MusicOrganizer;
wxString MusicOrganizer::stFindResourceFile(const char * name, const char * extension);
#endif

BEGIN_EVENT_TABLE ( OptionsDialog, wxFrame )
    EVT_BUTTON(ID_SAVE,OptionsDialog::save)
    EVT_BUTTON(ID_CANCEL,OptionsDialog::cancel)
    EVT_BUTTON(ID_PLAYER,OptionsDialog::choosePlayer)
    EVT_TEXT(ID_REPLACE,OptionsDialog::textWriting)
    EVT_RADIOBUTTON(ID_RBPLAYER, OptionsDialog::onRadioButton)
END_EVENT_TABLE()

DirLister::DirLister(wxArrayString* files)  {
    m_files=files;//will contain the folders found
}

wxDirTraverseResult DirLister::OnFile(const wxString&  WXUNUSED(filename)){
    return wxDIR_CONTINUE;//this class don't interract with files
}

wxDirTraverseResult DirLister::OnDir(const wxString& dirname){
    m_files->Add(dirname);
    return wxDIR_IGNORE;//disable recursivity
}


/****************************************************************************************************
 *****************************************************************************************************/

OptionsDialog::OptionsDialog( wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, long style ) : wxFrame( parent, id, title, pos, size, style )
{
	this->SetSizeHints( wxDefaultSize, wxDefaultSize );
   //set the icon of the frame
#ifndef __WXMSW__
	wxIcon icone=wxIcon ( icon_xpm );
#else
    wxIcon icone=wxIcon(_T("A"));
#endif
    SetIcon(icone);
    //set the imagelist for the toolbook
    #ifdef __WXMSW__
    wxImageList* columnImageList = new wxImageList(64,64,true);
    #else
    wxImageList* columnImageList = new wxImageList(64,64,false);
    #endif
    columnImageList->Add(wxBitmap(op_gen_xpm));
    columnImageList->Add(wxBitmap(op_adv_xpm));

	wxBoxSizer* bSizer1;
	bSizer1 = new wxBoxSizer( wxVERTICAL );
	toolBook = new wxToolbook( this, wxID_ANY, wxDefaultPosition, wxDefaultSize);
    toolBook->SetImageList(columnImageList);
    toolBook->SetBackgroundColour(wxColour(255,255,255));
	wxPanel *m_panel1 = new wxPanel( toolBook, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
	wxPanel *m_panel3 = new wxPanel( this, wxID_ANY, wxDefaultPosition, wxSize( -1,40 ), wxTAB_TRAVERSAL );
	wxPanel *m_panel2 = new wxPanel( toolBook, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL );
	wxBoxSizer* bSizer3;
	bSizer3 = new wxBoxSizer( wxVERTICAL );
    wxStaticBox *sb1 = new wxStaticBox( m_panel1, wxID_ANY, _("Replace character") );
    wxStaticBox *sb2 = new wxStaticBox( m_panel1, wxID_ANY, _("Capitalize") );
    wxStaticBox *sb3 = new wxStaticBox( m_panel2, wxID_ANY, _("Language") );
    wxStaticBox *sb4 = new wxStaticBox( m_panel2, wxID_ANY, _("Player") );
	wxStaticBoxSizer* sbSizer2;
	sbSizer2 = new wxStaticBoxSizer( new wxStaticBox( m_panel1, wxID_ANY, _("Global") ), wxVERTICAL );

	saveDir = new wxCheckBox(m_panel1,wxID_ANY,_("Remember the current directory when closing"));
    delEmptyFolders = new wxCheckBox(m_panel1,wxID_ANY,_("Delete empty folders after moving"));
    saveTabs = new wxCheckBox(m_panel1,wxID_ANY,_("Save tabs content"));
    recursiveScan = new wxCheckBox(m_panel1,wxID_ANY,_("Recurcive folders scan"));
    directorySort = new wxCheckBox(m_panel1,wxID_ANY,_("Sort by directory"));
    saveHist = new wxCheckBox(m_panel1,wxID_ANY,_("Keep an history about the directories browsed"));

    btnCancel = new wxButton(m_panel3,ID_CANCEL,_("Cancel"));
    btnSave = new wxButton(m_panel3,ID_SAVE,_("Save"));
    btnPlayer = new wxButton(m_panel2,ID_PLAYER,_("Choose ..."));

    langs = new wxChoice(m_panel2,wxID_ANY);

    radioDefaultPlayer = new wxRadioButton(m_panel2, ID_RBPLAYER,_("Default Player"));
    radioPersoPlayer = new wxRadioButton(m_panel2, ID_RBPLAYER,_("Custom Player"));

    customPlayer = new wxTextCtrl(m_panel2,ID_PLAYER_TXT,wxEmptyString,wxDefaultPosition,wxSize(200,-1));
    defaultReplaceChar = new wxTextCtrl(m_panel1,ID_REPLACE,wxEmptyString);
    ucSymbols = new wxTextCtrl(m_panel1,ID_CAPITALIZE,wxEmptyString,wxDefaultPosition,wxSize(270,-1));

    wxStaticText* txtReplace = new wxStaticText(m_panel1,wxID_ANY,_("Replace character")),
                  *txtCapitalize = new wxStaticText(m_panel1,wxID_ANY,_("Capitalize the next letter after the following symbols")),
                  *txtLng = new wxStaticText(m_panel2,wxID_ANY,_("Langage of the application (needs a restart to work)"));

	//wxBoxSizer* bSizer4;
	//bSizer4 = new wxBoxSizer( wxHORIZONTAL );

	wxStaticBoxSizer* sbSizer3;
	sbSizer3 = new wxStaticBoxSizer(sb2, wxVERTICAL );

	txtReplace->Wrap( -1 );
	txtCapitalize->Wrap( -1 );

	sbSizer3->Add( txtCapitalize, 0, wxALL, 5 );
	sbSizer3->Add( ucSymbols, 0, wxALL, 5 );

	//bSizer4->Add( sbSizer3, 1, wxEXPAND, 5 );

	wxStaticBoxSizer* sbSizer6;
	sbSizer6 = new wxStaticBoxSizer(sb1, wxHORIZONTAL );
	sbSizer6->Add( txtReplace, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );
	sbSizer6->Add( defaultReplaceChar, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );

	sbSizer2->Add( saveDir, 0, wxALL, 5 );
	sbSizer2->Add( delEmptyFolders, 0, wxALL, 5 );
	sbSizer2->Add( saveTabs, 0, wxALL, 5 );
	sbSizer2->Add( recursiveScan, 0, wxALL, 5 );
	sbSizer2->Add( directorySort, 0, wxALL, 5 );
	sbSizer2->Add( saveHist, 0, wxALL, 5 );
	sbSizer2->Add( sbSizer6, 1, wxEXPAND, 5 );

	bSizer3->Add( sbSizer2, 1, wxEXPAND, 5 );
	bSizer3->Add( sbSizer3, 0, wxEXPAND, 5 );

	m_panel1->SetSizer( bSizer3 );
	m_panel1->Layout();
	bSizer3->Fit( m_panel1 );
	toolBook->AddPage( m_panel1, _("General"), true ,0);
	wxBoxSizer* bSizer5;
	bSizer5 = new wxBoxSizer( wxVERTICAL );

	wxStaticBoxSizer* sbSizer5;
	sbSizer5 = new wxStaticBoxSizer( sb3, wxVERTICAL );

	txtLng->Wrap( -1 );
	sbSizer5->Add( txtLng, 0, wxALL, 5 );

	sbSizer5->Add( langs, 0, wxALL, 5 );

	bSizer5->Add( sbSizer5, 0, wxEXPAND, 5 );

	wxStaticBoxSizer* sbSizer4;
	sbSizer4 = new wxStaticBoxSizer( sb4, wxVERTICAL );

	sbSizer4->Add( radioDefaultPlayer, 0, wxALL, 5 );

	wxBoxSizer* bSizer6;
	bSizer6 = new wxBoxSizer( wxHORIZONTAL );

	bSizer6->Add( radioPersoPlayer, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );

	bSizer6->Add( customPlayer, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );

	bSizer6->Add( btnPlayer, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5 );

	sbSizer4->Add( bSizer6, 0, wxEXPAND, 5 );

	bSizer5->Add( sbSizer4, 1, wxEXPAND, 5 );

	m_panel2->SetSizer( bSizer5 );
	m_panel2->Layout();
	bSizer5->Fit( m_panel2 );
    toolBook->AddPage( m_panel2, _("Advanced"), false ,1);

	bSizer1->Add( toolBook, 1, wxEXPAND | wxALL, 1 );

	wxBoxSizer* bSizer13;
	bSizer13 = new wxBoxSizer( wxHORIZONTAL );

	wxBoxSizer* bSizer14;
	bSizer14 = new wxBoxSizer( wxVERTICAL );

	bSizer13->Add( bSizer14, 1, wxEXPAND, 5 );

	bSizer13->Add( btnSave, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT|wxALL, 5 );

	bSizer13->Add( btnCancel, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_RIGHT|wxALL, 5 );


	m_panel3->SetSizer( bSizer13 );
	m_panel3->Layout();
	bSizer1->Add( m_panel3, 0, wxEXPAND | wxALL, 0 );

	this->SetSizer( bSizer1 );
	this->Layout();
	bSizer1->Fit(this);

//get all available languages
	wxString pod=_T ( "./po" ) ;
#ifdef __WXMAC__
    if(!wxDirExists( _T ( "./po" ) ))
      pod=MusicOrganizer::stFindResourceFile ( "po","" );
#endif

	langs->Append(_T("English"));
    langs->SetStringSelection(_T("English"));
    if(wxDirExists(pod)){//if their are some languages installaled
        wxDir po(pod);
        wxArrayString langsDispo;
        langsId.Clear();

        DirLister dl(&langsDispo);

        po.Traverse(dl);//get all subdirectories of po dir

        for(int i=0 ; i<langsDispo.Count() ; i ++){//for each subdirectories
             wxString url=langsDispo[i];

             //get the name of the language
             langsDispo[i]=langsDispo[i].Mid(langsDispo[i].Find(wxFileName::GetPathSeparator(),true)+1);
             //wxMessageBox(url+wxString::Format(_T("%cmusicorganizer.mo"),wxFileName::GetPathSeparator()));
             const wxLanguageInfo* li= wxLocale::FindLanguageInfo(langsDispo[i]);
             //check if the translation is in the dir
             if(li!=NULL && wxFileName::FileExists(url+wxString::Format(_T("%cmusicorganizer.mo"),wxFileName::GetPathSeparator()))){
                  langs->Append(li->Description);//retreive the english name of the language
                  langsId.Add(li->Language);
                  if(OptionsHandler::getOption(_T("/app/language"),wxLocale::GetSystemLanguage())==li->Language)
                     langs->SetStringSelection(li->Description);
             }
        }
    }

    //read options
    wxRegEx speCharExp(wxT("[\\:*?\"<>|/]"));
    wxString rep=OptionsHandler::getOption(_T("/notebook/replace_char"),wxString(_T("_")));
    //remove specials chars for textWriting
    speCharExp.Replace(&rep,wxString(_T("")),0);
    bool sd = OptionsHandler::getOption(_T("/tree/save_dir"),true);
    bool ds = OptionsHandler::getOption(_T("/list/sort_by_dir"),true);
    bool sup = OptionsHandler::getOption(_T("/notebook/clean_dir"),true);
    bool save = OptionsHandler::getOption(_T("/notebook/save_content"),true);
    bool rec = OptionsHandler::getOption(_T("/tree/recursive"),true);
    bool sh = OptionsHandler::getOption(_T("/tree/has_history"),true);
    wxString play = OptionsHandler::getOption(_T("/list/player"),wxString(_T("-1")));
    wxString symbols = OptionsHandler::getOption(_T("/notebook/capitalize"),wxString(wxT("0123456789{}()[].~`!@#$%^&*-_+=|\\:;\"<>,?/")));
    //write in the dialog the options values
    defaultReplaceChar->SetValue(rep);
    ucSymbols->SetValue(symbols);
    saveDir->SetValue(sd);
    directorySort->SetValue(ds);
    delEmptyFolders->SetValue(sup);
    saveTabs->SetValue(save);
    recursiveScan->SetValue(rec);
    saveHist->SetValue(sh);
    radioDefaultPlayer->SetValue((play==_T("-1")));
    if(play!=_T("-1")){//default player
        customPlayer->SetValue(play);
        radioDefaultPlayer->SetValue(false);
        radioPersoPlayer->SetValue(true);
    }else{
        customPlayer->Enable(false);
        btnPlayer->Enable(false);
    }
}

/* ************************************************************************************* */

OptionsDialog::~OptionsDialog(){}

/* ************************************************************************************* */

void OptionsDialog::choosePlayer(wxCommandEvent& e){
#ifdef __WXMSW__
    //for windows a binary program end by exe, a filter can be made
    wxString wildcard(wxString(_("Executable files"))+wxString(_T(" (*.exe) |*.exe")));
#else //under *nix binary programs have no particular extensions
    wxString wildcard(_T("*"));
#endif

    wxFileDialog pl(this,_("Please choose your player"),_T(""),_T(""),wildcard,wxFD_OPEN | wxFD_FILE_MUST_EXIST);
    if(pl.ShowModal()==wxID_OK)
        customPlayer->SetValue(pl.GetPath());

}

/* ************************************************************************************* */

void OptionsDialog::save(wxCommandEvent& e){
    //save all options
    OptionsHandler::setOption(_T("/tree/has_history"),saveHist->IsChecked());
    if(!saveHist->IsChecked()){
        for(int i=1 ; i<=10 ; i++)//we clean history
             OptionsHandler::setOption(wxString::Format(_T("/tree/history/path%d"),i),wxString(_T("")));
             OptionsHandler::setOption(_T("/tree/history/index"), 0);
    }else{
             OptionsHandler::setOption(_T("/tree/history/path1"), OptionsHandler::getOption(_T("/tree/path")));
             OptionsHandler::setOption(_T("/tree/history/index"), 1);
    }

    wxString lngChoice=langs->GetStringSelection();
    int idLng;
    if(lngChoice==_T("English")){
         idLng=wxLANGUAGE_UNKNOWN;//nothing will be translated
    }else{
         wxArrayString values=langs->GetStrings();
         int index=values.Index(lngChoice);
         idLng=langsId[index-1];
    }
    OptionsHandler::setOption(_T("/app/language"),idLng);

    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, MusicOrganizer::ID_HIST);
    event.SetInt(1);
    wxPostEvent( GetParent(), event );
    OptionsHandler::setOption(_T("/notebook/replace_char"),defaultReplaceChar->GetValue());
    OptionsHandler::setOption(_T("/tree/save_dir"),saveDir->IsChecked());
    OptionsHandler::setOption(_T("/list/sort_by_dir"),directorySort->IsChecked());
    OptionsHandler::setOption(_T("/notebook/clean_dir"),delEmptyFolders->IsChecked());
    OptionsHandler::setOption(_T("/notebook/save_content"),saveTabs->IsChecked());
    OptionsHandler::setOption(_T("/tree/recursive"),recursiveScan->IsChecked());
    OptionsHandler::setOption(_T("/notebook/capitalize"),ucSymbols->GetValue());
    if(radioDefaultPlayer->GetValue()==1)//default player
        OptionsHandler::setOption(_T("/list/player"),wxString(_T("-1")));
    else
        OptionsHandler::setOption(_T("/list/player"),customPlayer->GetValue());
    Destroy();
}

/* ************************************************************************************* */

void OptionsDialog::cancel(wxCommandEvent& e){
    //don't save anything
    Destroy();
}

/* ************************************************************************************* */

void OptionsDialog::textWriting(wxCommandEvent& event){
    //disable typping of unallowed chars for the replace char
    wxString s = defaultReplaceChar->GetValue();
    wxRegEx invalid(wxT("[\\:*?\"<>|]"));
    if (invalid.Matches(s)){
        defaultReplaceChar->SetValue(text);
        defaultReplaceChar->SetInsertionPointEnd();
        s = text;
    }else{
        text = s;
    }
}

/* ************************************************************************************* */

void OptionsDialog::onRadioButton(wxCommandEvent& e){
    //if default player then the text box for custom player is disable and vice versa
    customPlayer->Enable(radioDefaultPlayer->GetValue()!=1);
    btnPlayer->Enable(radioDefaultPlayer->GetValue()!=1);
}
