/***************************************************************************
 *   Name:      Options_Handler.h                                          *
 *   Purpose:   Defines Application Frame                                  *
 *   Copyright (C) 2007 by Jackie Team                                     *
 *   jackie.team@gmail.com                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef OPTIONS_HANDLER_H
#define OPTIONS_HANDLER_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/config.h> 
#include <map>

/**
 *this class is used to abstract option handling,
 *saving and restoring options whith default value
 */
class OptionsHandler {
    public:
         /**
          * read the options and store it in memory
          */
         static void loadOptions();

         /**
          * get the value of an option
          * @param  wxString the name of the option
          * @param  wxString default value if the option doesn't exist yet
          * @return the value, "" if the option doesn't exist
          */
         static wxString getOption(wxString,wxString=_T(""));
         static int getOption(wxString,int);
         static double getOption(wxString,double);
         static bool getOption(wxString,bool);

         /**
          * define an option
          * @param  wxString the name of the option
          * @param  wxString the value of the option
          */
         static void setOption(wxString,wxString);
         static void setOption(wxString,int);
         static void setOption(wxString,double);
         static void setOption(wxString,bool);

    private:
        OptionsHandler();//it mustn't be instanciated
        static wxConfigBase* config;
};

#endif
