/***************************************************************************
 * Name:      checklistctrl.cpp                                            *
 * Purpose:   Code for the listctrl with checkbox                          *
 * Author:    Jackie Team (jackie.team@gmail.com)                          *
 * Created:   2007-11-26                                                   *
 * Copyright: Jackie Team (http://www.musicorganizer.org)                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// Include XPM icons
#include "images/checked.xpm"
#include "images/unchecked.xpm"

#include "checklistctrl.h"

	/*
	 * wxCheckListCtrl
	 */

	IMPLEMENT_CLASS(wxCheckListCtrl, wxListCtrl)

	BEGIN_EVENT_TABLE(wxCheckListCtrl, wxListCtrl)
		EVT_MOUSE_EVENTS(wxCheckListCtrl::OnMouseEvent)
		EVT_CONTEXT_MENU(wxCheckListCtrl::onContextMenu)
		EVT_MENU(ID_CHECK,wxCheckListCtrl::onCheck)
		EVT_MENU(ID_UNCHECK,wxCheckListCtrl::onUncheck)
	END_EVENT_TABLE()

	wxCheckListCtrl::wxCheckListCtrl(wxWindow* parent, wxWindowID id, const wxPoint& pt,
	const wxSize& sz, long style):
	wxListCtrl(parent, id, pt, sz, style)
	{
		LoadIcons();
	}


	/// Load the icons
	bool wxCheckListCtrl::LoadIcons()
	{
		//FIXME : Manual delete need
		m_imageList = new wxImageList(16, 16, true);
		AssignImageList(m_imageList, wxIMAGE_LIST_SMALL);

		m_imageList->Add(wxIcon(checked_xpm));
		m_imageList->Add(wxIcon(unchecked_xpm));

		return true;
	}

	void wxCheckListCtrl::onContextMenu(wxContextMenuEvent& event){
		wxPoint point = event.GetPosition();
		// If from keyboard
		if (point.x == -1 && point.y == -1) {
			wxSize size = GetSize();
			point.x = size.x / 2;
			point.y = size.y / 2;
		} else {
			point = ScreenToClient(point);
		}
		wxMenu menu;

		menu.Append(ID_CHECK, _("&Check selected"));
		menu.Append(ID_UNCHECK, _("&Uncheck selected"));
		if(GetSelectedItemCount()==0){
			menu.Enable(ID_CHECK,false);
			menu.Enable(ID_UNCHECK,false);
		}

		PopupMenu(&menu, point.x, point.y);
	}

	long wxCheckListCtrl::getSelection(int nextSel){
		return GetNextItem(nextSel, wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED);
	}

	void wxCheckListCtrl::onCheck(wxCommandEvent& WXUNUSED(event)){
		long nextSel=getSelection(-1);
		while(nextSel!=-1){
			SetIcon(nextSel,0);
			nextSel=getSelection(nextSel);
		}

	}

	void wxCheckListCtrl::onUncheck(wxCommandEvent& WXUNUSED(event)){
		long nextSel=getSelection(-1);
		while(nextSel!=-1){
			SetIcon(nextSel,1);
			nextSel=getSelection(nextSel);
		}
	}



	wxCheckListCtrl::~wxCheckListCtrl()
	{
	}


	/// Set the appropriate icon
	bool wxCheckListCtrl::SetIcon(long& item,int state)
	{
		bool l_ret = true;

		wxListItem l_item;
		l_item.SetId(item);
        	l_item.SetMask(wxLIST_MASK_IMAGE);
		GetItem(l_item);

		if ( state==-1 )  //reverse current state
		{
			if ( l_item.GetImage() == UNCHECK_ICON )
			{
				SetItemImage ( item, CHECK_ICON );
			}
			else
			{
				SetItemImage ( item, UNCHECK_ICON );
			}
		}
		else if ( state==0 )
		{
			SetItemImage ( item, CHECK_ICON );
		}
		else
		{
			SetItemImage ( item, UNCHECK_ICON );
		}
		return l_ret;
	}


	void wxCheckListCtrl::OnMouseEvent(wxMouseEvent& event)
	{
		int l_flags = 0;
		long l_subitem;
		long l_item = HitTest(wxPoint(event.GetX(), event.GetY()), l_flags, &l_subitem);
		if (event.LeftDown())
		{
			if (l_flags & wxLIST_HITTEST_ONITEMICON)
			{
				SetIcon(l_item);
			}
		}

		event.Skip();
	}


	void wxCheckListCtrl::OnKeyDown(wxKeyEvent& event)
	{
		int l_flags = 0;
		long l_subitem;
		long l_item = HitTest(wxPoint(event.GetX(), event.GetY()), l_flags, &l_subitem);

		if (event.GetKeyCode() == WXK_SPACE)
		{
			SetIcon(l_item);
		}
		else
		{
			event.Skip();
		}
	}

	bool wxCheckListCtrl::IsChecked(long item)
	{
		wxListItem l_item;
		l_item.SetId(item);
		l_item.SetMask(wxLIST_MASK_IMAGE);
		GetItem(l_item);
		if(l_item.GetImage() == 0)
			return true;
		else
			return false;
	}


