/***************************************************************************
 * Name:      compents.h                                                   *
 * Purpose:   Defines Main Panel and Components Classes                    *
 * Author:    Jackie Team (jackie.team@gmail.com)                          *
 * Created:   2007-12-07                                                   *
 * Copyright: Jackie Team (http://www.musicorganizer.org)                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef COMPONENTS_H_INCLUDED
#define COMPONENTS_H_INCLUDED

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

//a lot of includes...
#include <wx/listctrl.h>
#include <wx/panel.h>
#include <wx/notebook.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/combobox.h>
#include <wx/arrstr.h>
#include <wx/bmpbuttn.h>
#include <wx/bitmap.h>
#include <wx/button.h>
#include <wx/imagpng.h>
#include <wx/image.h>
#include <wx/textctrl.h>
#include <wx/colour.h>
#include <wx/panel.h>
#include <wx/dirctrl.h>
#include <wx/listctrl.h>
#include <wx/radiobox.h>
#include <wx/progdlg.h>
#include <wx/dirdlg.h>
#include <wx/file.h>
#include <wx/tokenzr.h>
#include <wx/frame.h>
#include <wx/checklst.h>
#include <wx/popupwin.h>
#include <wx/regex.h>
#include <wx/thread.h>
#include <wx/dir.h>
#include <wx/statbox.h>
#include <wx/checkbox.h>
#include <wx/encconv.h>
#include <wx/dynarray.h>
#include <wx/mimetype.h>
#include <wx/choicebk.h>
#include <wx/filename.h>
#include <wx/minifram.h>
#include <wx/dnd.h>
#include <vector>
#include <map>
#include <algorithm>
#include "Options_Handler.h"
#include "options_Dialog.h"
#include "checklistctrl.h"

//List of ID3V1 genres
const wxString GENRES[]={_T("Blues"),_T("ClassicRock"),_T("Country"),_T("Dance"),_T("Disco"),_T("Funk"),_T("Grunge"),_T("Hip-Hop"),_T("Jazz"),
_T("Metal"),_T("NewAge"),_T("Oldies"),_T("Other"),_T("Pop"),_T("R&B"),_T("Rap"),_T("Reggae"),_T("Rock"),_T("Techno"),_T("Industrial"),_T("Alternative"),
_T("Ska"),_T("DeathMetal"),_T("Pranks"),_T("Soundtrack"),_T("Euro-Techno"),_T("Ambient"),_T("Trip-Hop"),_T("Vocal"),_T("Jazz+Funk"),_T("Fusion"),
_T("Trance"),_T("Classical"),_T("Instrumental"),_T("Acid"),_T("House"),_T("Game"),_T("SoundClip"),_T("Gospel"),_T("Noise"),_T("AlternRock"),_T("Bass"),
_T("Soul"),_T("Punk"),_T("Space"),_T("Meditative"),_T("InstrumentalPop"),_T("InstrumentalRock"),_T("Ethnic"),_T("Gothic"),_T("Darkwave"),
_T("Techno-Industrial"),_T("Electronic"),_T("Pop-Folk"),_T("Eurodance"),_T("Dream"),_T("SouthernRock"),_T("Comedy"),_T("Cult"),_T("Gangsta"),
_T("Top40"),_T("ChristianRap"),_T("Pop/Funk"),_T("Jungle"),_T("NativeAmerican"),_T("Cabaret"),_T("NewWave"),_T("Psychadelic"),_T("Rave"),_T("Showtunes"),
_T("Trailer"),_T("Lo-Fi"),_T("Tribal"),_T("AcidPunk"),_T("AcidJazz"),_T("Polka"),_T("Retro"),_T("Musical"),_T("Rock & Roll"),_T("HardRock"),_T("Folk"),
_T("Folk-Rock"),_T("NationalFolk"),_T("Swing"),_T("FastFusion"),_T("Bebob"),_T("Latin"),_T("Revival"),_T("Celtic"),_T("Bluegrass"),_T("Avantgarde"),
_T("GothicRock"),_T("ProgressiveRock"),_T("PsychedelicRock"),_T("SymphonicRock"),_T("SlowRock"),_T("BigBand"),_T("Chorus"),_T("EasyListening"),
_T("Acoustic"), _T("Humour"),_T("Speech"),_T("Chanson"),_T("Opera"),_T("ChamberMusic"),_T("Sonata"),_T("Symphony"),_T("BootyBass"),_T("Primus"),
_T("PornGroove"),_T("Satire"),_T("SlowJam"),_T("Club"),_T("Tango"),_T("Samba"),_T("Folklore"),_T("Ballad"),_T("PowerBallad"),_T("RhythmicSoul"),_T("Freestyle"),
_T("Duet"),_T("PunkRock"),_T("DrumSolo"),_T("Acapella"),_T("Euro-House"),_T("DanceHall")};

//classes prototypes
class MonArbo;
class MaListe;
class MonChoicebook;
class MonBook;
class ComboPath;

/**
 *Class to retrieve all mp3 files recursively or not in folder
 */
class wxDirTraverserMp3 : public wxDirTraverser
{
    public:
        /**
         *Constructor
         *@param wxArrayString* a pointer to the variable which will contains all files found (if any)
         *@param bool if true the scan will be recursive
         */
        wxDirTraverserMp3(wxArrayString* files,bool recursive);

        virtual wxDirTraverseResult OnFile(const wxString& filename);

        virtual wxDirTraverseResult OnDir(const wxString& WXUNUSED(dirname));

        /**
         *Abbort immediately the scan
         */
        void stopScan();

    private:
        wxArrayString* m_files;
        bool stop,rec;
};

/**
 *Class which copy or move files in his own thread
 */
class CopyMoveThread : public wxThread{
    public:
       /**
        *Constructor
        * @param l the list of files
        * @param b the notebook
        * @param d the directory in which the files will be copied/moved
        * @param move if false just copy, else move files
        */
        CopyMoveThread(MaListe* l,MonBook* b,wxString& d,bool move=false);
        virtual void *Entry();
        virtual void OnExit();
        //getters
        long getProg();
        int getErrors();
        /**
         *Cause the end of the thread and its task
         */
        void end();

    private:
        enum{ID_THREAD=756};
        MaListe* ml;
        MonBook* mb;
        bool stop,mv;
        long prog;
        int errors;
        wxString dossier;

};

/**
 *Class which copy files dropped on MonArbo
 */
class CopyDropThread : public wxThread{
    public:
       /**
        *Constructor
        * @param l the list of files
        * @param b the notebook
        * @param d the directory in which the files will be copied/moved
        * @param move if false just copy, else move files
        */
        CopyDropThread(MonArbo* a,wxString data,wxString pt);
        virtual void *Entry();
        virtual void OnExit();
        /**
         *Cause the end of the thread and its task
         */
        void end();
        //getter
        long getProg();

    private:
        enum{ID_THREAD=765};
        MonArbo* ma;
        bool stop;
        long prog;
        int errors;
        wxString files;
        wxString path;

};

/**
 *A modal frame
 */
class wxModalFrame : public wxFrame{
    private:
        void onClose(wxCloseEvent& event);
        DECLARE_EVENT_TABLE()
    public:
        /**
         *Same constructor as wxFrame see wxFrame for further informations
         */
        wxModalFrame(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE, const wxString& name = _T("frame"));
};

/**
 *A combobox used by MonBook to store tags of selected items
 */
class ComboBox : public wxComboBox{
    public:
        /**
         *push a string item in the combobox
         *@param wxString the string to push
         */
        void addComboBoxItem(wxString);
        /**
         *remove a string item of the combobox
         *@param wxString the string item to remove
         */
        void removeComboBoxItem(wxString);

        /**
         *Constructor, same as wxCombobox except their is one more before them, see wxCombobox doc for the others
         *@param MonBool* the MonBook associated with the combox, it may not be the parent
         */
        ComboBox(MonBook* b, wxWindow* parent, wxWindowID id, const wxString& value,const wxPoint& pos, const wxSize& size, const wxArrayString& choices, long style=0);

        /**
         *Remove all item of the combobox
         */
        void clearAll();
    protected:
        void onGainFocus(wxFocusEvent& e);
        void onLooseFocus(wxFocusEvent& e);
    private:
        std::map<wxString, long> duplicateMap;//le 'long' correspond au nombre d'occurence du 'wxString'
        MonBook* mb;
        DECLARE_EVENT_TABLE()
};

/**
 *A list box to display completion result and let the user select a value
 */
class completionList : public wxListBox{
    public:
        /**
         *Constructor
         *@param ComboPath* the comboPath associated
         *@param wxWindow* the parent
         *@param wxWindowID the ID of the listbox
         *@param wxPoint the coordinates of the object
         *@param wxSize the dimension of the object
         *@param wxString* the initial content
         *@param int the style of the list
         */
        completionList(ComboPath *, wxWindow*, wxWindowID ,wxPoint, wxSize,int,wxString*,long int);

    private:
        void onGainFocus(wxFocusEvent& e);
        void onLooseFocus(wxFocusEvent& e);
        void onClick(wxMouseEvent& e);
        void onClick2(wxCommandEvent& e);
        ComboPath* cp;
        DECLARE_EVENT_TABLE()
};

/**
 *A combox which show the current path and allow the user to change path by typing it
 */
class ComboPath : public ComboBox{
    public:
        ComboPath(MonBook* b,wxWindow* parent, wxWindowID id, const wxString& value,const wxPoint& pos, const wxSize& size, const wxArrayString& choices, long style=0);
        //setters
        void setMonArbo(MonArbo *a);
        void setPath(wxString);
    private:
        void onGainFocus(wxFocusEvent& e);
        void onLooseFocus(wxFocusEvent& e);
        void onEnter(wxCommandEvent& e);
        void onText(wxCommandEvent& e);
        void onKey(wxKeyEvent& e);
        MonArbo* ma;
        wxArrayString files;
        #ifndef __WXMAC__
        wxPopupWindow* pw;
        #else
        wxMiniFrame* pw;//pop-up window is not yet implemented for Mac OS
        #endif

        completionList* cl;
        wxString oldPathWritten;
        DECLARE_EVENT_TABLE()
};


/**
 *The notebook which contains button and combobox to modifie the tags/filenames
 */
class MonBook : public wxNotebook{
    private:
        wxPanel* panelPage1;
        wxPanel* panelPage2;
        wxPanel* panelPage3;
        wxPanel* panelPage4;
        wxPanel* panelPage5;
        ComboBox* comboTitre;
        ComboBox* comboInterprete;
        ComboBox* comboAlbum;
        ComboBox* comboAnne;
        ComboBox* comboPiste;
        ComboBox* comboGenre;
        wxBitmapButton* button_save;
        ComboBox* comboTitre2;
        ComboBox* comboInterprete2;
        ComboBox* comboAlbum2;
        ComboBox* comboAnne2;
        ComboBox* comboPiste2;
        ComboBox* comboGenre2;
        wxBitmapButton* button_save2;
        ComboBox* comboOrg;
        wxButton* btnCopierIdv;
        wxButton* btnCopier;
        wxButton* btnDeplacer;
        wxButton* btnQuickDeplacer;
        wxStaticText* quickDir;
        //wxButton* btnCleanID3V2;
        wxTextCtrl* DescrSaisie;
        wxStaticText* infoSaisie;
        ComboBox* comboNomFic;
        wxButton* btnApercu;
        wxTextCtrl* DescrSaisie2;
        wxCheckBox* chkTitre;
        wxCheckBox* chkAlbum;
        wxCheckBox* chkAnne;
        wxCheckBox* chkGenre;
        wxCheckBox* chkPiste;
        wxCheckBox* chkArtiste;
        wxCheckBox* chkNoEmpty;
        wxChoice* chooseCase;
        //--_Page_5_--
        wxTextCtrl* txtTitreGen;
        wxTextCtrl* txtInterpreteGen;
        wxTextCtrl* txtAlbumGen;
        ComboBox* comboAnneGen;
        wxTextCtrl* txtPisteGen;
        ComboBox* comboGenreGen;
        wxButton* btnGen;
        wxButton* btnAutoTrack;
        wxCheckBox* chkGenAll;
        wxCheckBox* chkNoEmptyGen;
        wxRadioBox* radioGen;
        wxTextCtrl* txtSeparator;
        //--_Fin_page_5_--
        wxString text1;
        wxString text2;
        wxButton* btnAnnuler;
        wxButton* btnValider;
        wxRadioBox* radio;
        wxCheckListCtrl* LstBox;
        wxModalFrame* containList;
        wxArrayString listeNoms,listeNoms2;
        wxProgressDialog* progBar;
        CopyMoveThread* thread;
        enum{ID_SAVE=637,ID_GEN_TRACK,ID_GEN,ID_COPYTAG,ID_SAVE2,ID_CLEAN,ID_COPIER,ID_DEPLACER,ID_QUICK_DEPLACER,ID_APPERCU,ID_COMBOORG,ID_FIC,ID_TRACK,ID_ANNEE,ID_THREAD=756,ID_VALIDER_GEN,MAKE_UPPER=2,MAKE_LOWER=1,MAKE_FWL_UPPER=3};
        MonChoicebook* cb;
        wxComboBox* comboFormatPiste;
        MonArbo* arbo;
        void button_click(wxCommandEvent&);
        void button_clickV1(wxCommandEvent&);
        void btnCopyClick(wxCommandEvent&);
        void btnMoveClick(wxCommandEvent&);
        void btnQuickMoveClick(wxCommandEvent&);
        void btnCopyTag(wxCommandEvent&);
        //void btnCleanID3v2Click(wxCommandEvent& event);
        void btnApercuRename(wxCommandEvent&);
        void btnValiderClick(wxCommandEvent&);
        void btnValiderGenClick(wxCommandEvent&);
        void btnAnnulerClick(wxCommandEvent&);
        void textWriting(wxCommandEvent&);
        void textWriting2(wxCommandEvent&);
        void textWritingNumbers(wxCommandEvent&);
        void textWritingNumbers2(wxCommandEvent&);
        void cpyMoveThreadMsg(wxCommandEvent&);
        void generateTags(wxCommandEvent&);
        void gen(wxString&, wxString, wxString);
        void stopchars(wxKeyEvent& e);
        void generateTracks(wxCommandEvent& event);
        DECLARE_EVENT_TABLE()

    public:
        /**
         * Constructor
         * @param parent the parent of the component
         */
        MonBook(wxWindow *parent);
        //Update the overview of ComboOrg
        void updateDescrSaisie2();
        //Update the overview of ComboNomFic
        void updateDescrSaisie();
        //setters
        void setChoicebook(MonChoicebook* c);
        void setArbo(MonArbo* a);
        //getters
        ComboBox* getComboTitre(){ return comboTitre;}
        ComboBox* getComboInterprete(){ return comboInterprete;}
        ComboBox* getComboAlbum(){ return comboAlbum;}
        ComboBox* getComboAnne(){ return comboAnne;}
        ComboBox* getComboPiste(){ return comboPiste;}
        ComboBox* getComboGenre(){ return comboGenre;}
        ComboBox* getComboTitre2(){ return comboTitre2;}
        ComboBox* getComboInterprete2(){ return comboInterprete2;}
        ComboBox* getComboAlbum2(){ return comboAlbum2;}
        ComboBox* getComboAnne2(){ return comboAnne2;}
        ComboBox* getComboPiste2(){ return comboPiste2;}
        ComboBox* getComboGenre2(){ return comboGenre2;}
        wxComboBox* getComboOrg(){ return comboOrg;}
        MonChoicebook* getMonChoicebook(){return cb;}
        /**
         * Smart removing of non ascii characters
         * @param  s the string to clean
         * @param  replacement the replacement characters
         */
        void cleanStr(wxString&,wxString&);
        void showMsg(wxString,wxString);
        /**
         *Change the case of a string
         *@param wxString the string to work on
         *@param int if MAKE_FWL_UPPER Put first letters of each words of a string in uppercase and other chars in lowercase\n if MAKE_UPPER make the string in uppercase\n if MAKE_LOWER make the string in lowercase
         */
        wxString ucwords(const wxString&,int action);
        /**
         *Save the content of all tabs in option file
         */
        void saveTabs();
        /**
         *Restore the content previously saved of all tabs
         */
        void restoreTabs();

};

/**
 *The listCtrl to show id3V2 tags of files in current directory
 */
class MaListe : public wxListCtrl{
    private:
        MonBook* book;
        MonChoicebook* cb;
        MonArbo* arbo;
        int currentCol;
        bool desc,sortByFolder,is_key_pressed;
        wxImageList* columnImageList;
        void onContextMenu(wxContextMenuEvent& event);
        void touche(wxKeyEvent&);
        void toucheHaut(wxKeyEvent&);
        void onItemSelected(wxListEvent& event);
        void onItemDeselected(wxListEvent& event);
        void sort(wxListEvent& event);
        void onRefresh(wxCommandEvent& WXUNUSED(event));
        void onDel(wxCommandEvent& WXUNUSED(event));
        void onSel(wxCommandEvent& WXUNUSED(event));
        void onPlay(wxCommandEvent& WXUNUSED(event));
        void onReset(wxCommandEvent& event);
        void onDClick(wxListEvent& e);
        void onRColClick(wxListEvent& e);
        void onDragInit(wxListEvent& event);
        bool enable;
        enum{ID_PLAY};
        DECLARE_EVENT_TABLE()
        DECLARE_DYNAMIC_CLASS(MaListe)
    public:
        MaListe(wxWindow *parent, MonBook* b,MonChoicebook *c);
        MaListe(){}

        /**
         * Get the text of a specic column in a row
         * @param  long the index of the line
         * @param  int the column
         * @return the text found
         */
        wxString getRowText(long, int);
        /**
         * get the text of a specic column in the selected row
         * @param  int the column
         * @param  int the index of the previous selected row, may be -1 to select the first selected row
         * @return the text found
         */
        wxString getSelRowText(int , int );
        /**
         * get the index of the next selected row
         * @param  int the current index of the row, may be -1 a specic column in
         * @return the next selected row index or -1 if not found
         */
        long getSelection(int);

        /**
         *Set or remove a veto on key event
         *@param bool if true allow key even to be processed
         */
        void unblockKeyEvent(bool b=true);

        /**
         *Remove the current column header image
         */
        void removeColumnImage();
        /**
         * Send a MENU_EVT for the main frame to change the state of two edit menu items (enable or not)
         * @param  int the mode : -1 to disable all, 0 to enable all, 1 to enable just the first and 2 to enable just the second
         */
        void sendMenuMsg(int);
        /**
         * Launch the player using the options to play selected files
         */
        void playCurrent();
        /**
         * Save the columns width
         */
        void saveColumnsWidth();
        //setters
        void setEnableEvent(bool=true);
        void setCurCol(int);
        void setArbo(MonArbo* a);
        //getters
        int getCurrentCol(){return currentCol;}
        bool sortAscending(){return !desc;}
        enum{NAME_COL=0,TITLE_COL,ARTIST_COL,ALBUM_COL,YEAR_COL,GENRE_COL,TRACK_COL,LEN_COL,DIR_COL};

};

/**
 *The Auto Track window
 */
class TrackFrame : public wxFrame {
	private:
        MonChoicebook* choiceBook;
		wxPanel* m_panel6;
		wxStaticText* lblDescription;
		wxStaticText* m_staticText29;
		wxChoice* m_choice2;
		wxCheckBox* chkID3V1and2;
		wxCheckBox* chkCounter;
		wxButton* btnGenTrack;
		wxButton* btnCancelTrack;
		enum{ID_PISTE_GEN=4444,ID_CLOSE};
		void genTracks(wxCommandEvent&);
		void Close(wxCommandEvent& event);
		DECLARE_EVENT_TABLE()
	public:
		TrackFrame( wxWindow* parent,MonChoicebook *c, wxWindowID id = wxID_ANY, const wxString& title = _("Auto Track"), const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxSize( 400,300 ), long style = wxCAPTION|wxCLOSE_BOX|wxMINIMIZE_BOX|wxCLIP_CHILDREN|wxTAB_TRAVERSAL );

};/**
 *A pop-up window whith a waiting message and a cancel button
 */
#ifndef __WXMAC__
class WaitPopup : public wxPopupWindow{
#else
class WaitPopup : public wxMiniFrame{
#endif
    public:
        WaitPopup(MonArbo*);

    private:
        MonArbo* ma;
        //cancel the current scan
        void Cancel(wxCommandEvent& event);
        DECLARE_EVENT_TABLE()

};
/**
 *A thread which list the file whith mp3 extension in a directory
 */
class ScanThread : public wxThread{
    public:
       /**
        * The constructor
        * @param a
        * @param c the absolute path of the directory to scan
        * @param f the list of files fund
        * @param rec if true the scan will be recursive
        */
        ScanThread(MonArbo* a,wxString c,wxArrayString* f,bool rec=true);
        virtual void *Entry();
        virtual void OnExit();
        /**
         *Stop immediately the scanning
         */
        void Stop();

    private:
        enum{THREAD=1234,PULSE};
        MonArbo* ma;
        bool recursive;
        wxString chemin;
        wxArrayString* file;
        wxDirTraverserMp3* dtm;

};

/**
 *A list of directory on the computer
 */
class MonArbo : public wxGenericDirCtrl{
    private:
        enum{THREAD=1234};
        MonChoicebook* cb;
        MonBook* book;
        WaitPopup* wait;
        ScanThread* t;

        wxArrayString files;
        bool ignore_event;
        void change(wxTreeEvent& e);
        //this function is called when the ScanThread has done its task
        void endChange(wxCommandEvent& e);
        void dropThreadMsg(wxCommandEvent&);
        int oldindex;
        int nextindex;
        wxProgressDialog* progBar;
        CopyDropThread * cdt;
        enum {ID_DROPTHREAD=765};
        DECLARE_DYNAMIC_CLASS(MonArbo)
        DECLARE_EVENT_TABLE()
    public:
        /**
         *The constructor
         * @param wxWindow* the parent of the component (may be NULL)
         * @param MaListe* the wxListCtrl
         * @param MonBook* the wxNotebook
         */
        MonArbo(wxWindow *parent,MonChoicebook* c,MonBook* b);
        MonArbo(){}//for the dynmic class
        /**
         *Tell the scanThread to stop
         */
        void stopThread();

        /**
         * Define if the class will do something when an EVT_TREE_SEL_CHANGED is received
         * @param bool if true the class will ignore the event
         */
	void setIgnore(bool);
 
        /**
         *refresh the tree
         */
        void refresh();
        /**
         *Move into the history
         *@param d the number of "steps" to go, if <0 it will go backward otherwise it will go forward
         */
        void historyGo(int d);
        //getters
        CopyDropThread* getCdt(){return cdt;}
        wxProgressDialog* getProgBar(){return progBar;}
        MonChoicebook* getMonChoicebook(){return cb;}
        //setters
        void setProgBar(wxProgressDialog* pdial){progBar=pdial;}
        void setCdt(CopyDropThread* t){cdt=t;}
        void SetPath(const wxString& path);
};

/**
 *A drop target wich copy files dropped in the folder which are under the mouse when dropping on the tree
 */
class DropTargetCopie : public wxTextDropTarget{
public:
     DropTargetCopie(MonArbo* a);
     virtual bool OnDropText(wxCoord x, wxCoord y,const wxString& data);
     virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def);
     virtual void OnLeave();

private:
     MonArbo *ma;
     wxTreeItemId oldid;

};

/**
 *This choicebook allow user to select the tags version displayed
 */
class MonChoicebook : public wxChoicebook{
    public:
        MonChoicebook(wxWindow *parent,MonBook* b);
        MaListe* getMListeV1();
        MaListe* getMListeV2();
        MaListe* getCurrentList();
        MaListe* getOtherList();
        /**
         * Extract tags from a file and puts them into a new line
         * @param f the path to the file
         * @param index the index of the line to fill
         */
        void readAndShowInfos(wxString f,int index);

        /**
         * Select an item of the specified list
         * @param long the item index
         * @param MaListe* the list of the item
         * @param bool if true the item will be selected else it will be deselected
         */
        void selectItem(long index,MaListe* who,bool sel=true);
 
        /**
         * Sort the two lists
         * @param int the column to sort
         * @param int the column on which the lists are sorted
         * @param bool if true the sorting will be done in decreasing order else in increasing order
         * @param MaListe* the list frow wich the sorting will be done
         */
        void sortLists(int& currentCol,int oldCol,bool desc,MaListe* who);
        /**
         * remove the selected row(s)
         */
        void delCurrent();
        /**
         * tell if the current displayed list is the ID3V2 list
         * @return true if it is
         */
        bool isID3V2();
        /**
         * disable some menu to avoid global shortcuts conflict
         * @param bool if true action menu will be disabled otherwise they will be enabled
         */
        void disableMenus(bool=true);
    private:
        MaListe* mlisteV1;
        MaListe* mlisteV2;
        MonBook* book;
        void onChanged(wxChoicebookEvent& e);
        void fill_combo();
        DECLARE_EVENT_TABLE()

};

/**
 *The panel which contain the choicebook
 */
class ListCtrlPanel : public wxPanel{
    public:
        ListCtrlPanel(wxWindow *parent,MonBook* b);
        //getter
        MonChoicebook* getCb();
    private:
        MonChoicebook* cb;

};

#endif // COMPONENTS_H_INCLUDED
